### Multinomial Logit model estimation

### データファイルの読み込み
Data <- read.csv("Test.csv",header=T)
## データ数:Dataの行数を数える
hh <- nrow(Data)

##### Logit model の対数尤度関数の定義 #####
fr <- function(x) {
  ### パラメータの宣言：
  ## 定数項
  b1 <- x[1]
  b2 <- x[2]

  ## 所要時間
  d1 <- x[3]
  
  ## 女性ダミー
  f1 <- x[4]
  
  ## 対数尤度のための変数を宣言
  LL = 0
  
  ## 効用の確定項の計算
                #定数項                         #所要時間              #女性ダミー
  v1 <- b1*matrix(1,nrow = hh,ncol=1) + d1*( Data$所要時間1 ) + f1*Data$女性ダミー
  v2 <- b2*matrix(1,nrow = hh,ncol=1) + d1*( Data$所要時間2 )
  v3 <-                                 d1*( Data$所要時間3 )
  
  ### 選択確率の計算
  vehicle1 <- Data$選択1利用可能性 * exp(v1)
  vehicle2 <- Data$選択2利用可能性 * exp(v2)
  vehicle3 <- Data$選択3利用可能性 * exp(v3) 

  
  ## 分母となる，各々のexp(V)の和をつくる
  deno <- (vehicle1 + vehicle2 + vehicle3)

  ## それぞれの選択確率を計算する
  P1 <- vehicle1 / deno
  P2 <- vehicle2 / deno
  P3 <- vehicle3 / deno

  ## 選択確率が0になってしまった場合1にする(後で対数を取るため)
  P1 <- (P1 != 0)*P1 + (P1 == 0)
  P2 <- (P2 != 0)*P2 + (P2 == 0)
  P3 <- (P3 != 0)*P3 + (P3 == 0)

  ## 実際の選択結果
  C1 <- (Data$選択結果 == 1)
  C2 <- (Data$選択結果 == 2)
  C3 <- (Data$選択結果 == 3)

  ## 対数尤度の計算
  LL <- colSums(C1*log(P1) + C2*log(P2) + C3*log(P3) )
}

##### 対数尤度関数frの最大化#####

##パラメータの初期値を与える
b0 <- numeric(4)

##パラメータ値の最適化
res_NelderMead <-  optim(b0,fr, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1))
res_BFGS       <-  optim(b0,fr, method = "BFGS", hessian = TRUE, control=list(fnscale=-1))
res_CG         <-  optim(b0,fr, method = "CG", hessian = TRUE, control=list(fnscale=-1))
res_SANN       <-  optim(b0,fr, method = "SANN", hessian = TRUE, control=list(fnscale=-1))

##### 計算結果を表示する関数 #####
show <- function(res,b0){
  ## パラメータ推定値、ヘッセ行列
  b   <- res$par
  hhh <- res$hessian

  ## t値の計算
  tval <- b/sqrt(-diag(solve(hhh)))

  ## 初期尤度
  L0 <- hh * log(1/3)
  ## 最終尤度
  LL <- res$value

  ##### 結果の出力 #####
  ##パラメータ推定値
  print("パラメータ")
  print(b)
  ## t値
  print("t値")
  print(tval)
  ## ヘッセ行列
  print("ヘッセ行列")
  print(hhh)
  ## 初期尤度
  print("初期尤度")
  print(L0)
  ## 最終尤度
  print("最終尤度")
  print(LL)
  ##ρ^2値
  print("尤度比")
  print((L0-LL)/L0)
  ## 修正済ρ^2値
  print("補正済み尤度比")
  print((L0-(LL-length(b)))/L0)
}

####Nelder-Mead法####
show(res_NelderMead,b0)

####BFGS法####
show(res_BFGS,b0)

####CG法####
show(res_CG,b0)

####SANN法####
show(res_SANN,b0)
